#!/usr/local/bin/perl -wT
use CGI;
use CGI::Carp 'fatalsToBrowser';

# Usage: (for testing outside of the web browser)
# maligner model=<model> address=<addr> source=<source> format=[ps|pdf] 
#          conservation=[agree|infocon|kunk|kkno] 
#          showgn=[0|1] showpr=[0|1] showpb=[0|1]

my $MENTERIC_DIR = "/usr/local/menteric";

my $conf = "$MENTERIC_DIR/etc/maligner.conf";
my $base = "/var/tmp/bacteria";
my $tmp = "$base/$$";

$ENV{'PATH'} = "$MENTERIC_DIR/bin:/usr/local/bin:/usr/bin";
$ENV{'USER'} = 'menteric'; # dotplot puts this in a DSC comment.
$ENV{'HOME'} = '/usr/local/'; # acrobat distiller wants this.

umask(0000);
$|=1;

sub plumber { &cleanup ; exit(0); }
$SIG{'PIPE'} = \&plumber;
$SIG{'INT'} = \&plumber;
$SIG{'TERM'} = \&plumber;
$SIG{'HUP'} = \&plumber;

my $q = new CGI;
my $model = $q->param('model');
my $address = $q->param('address');
my $source = $q->param('source');
my $format = $q->param('format');
my $conservation = $q->param('conservation');
my $showgn = $q->param('showgn');
my $showpr = $q->param('showpr');
my $showpb = $q->param('showpb');


if  (!defined($showgn)) {
    $showgn = 0;
} elsif ($showgn=~/^\s*(1)\s*$/) {
    $showgn = $1;
} else {
    die "Illegal numerical value for the gene display option.";
}

if  (!defined($showpr)) {
    $showpr = 0;
} elsif ($showpr=~/^\s*(1)\s*$/) {
    $showpr = $1;
} else {
    die "Illegal numerical value for the promoter display option.";  
}

if  (!defined($showpb)) {
    $showpb = 0;
} elsif ($showpb=~/^\s*(1)\s*$/) {
    $showpb = $1;
} else {
    die "Illegal numerical value for the protein binding site display option.";
}

if ($model=~/^\s*([a-zA-Z]+)\s*$/) {
    $model = $1;
} else {
    die "Illegal character in the name for the model system.";
} 

my $start;
my $stop;
my @annotations;
my $prefix;

&set_model_globals($model) or die "Invalid model organism $model.";

sub set_model_globals { # model 

    local ($ref_model) = $1;
    local ($setm, $setf, $sett, $seta, $setp);
    local ($m, $f, $t, @a, $p); 

    $setm = $setf = $sett = $seta = $setp = 0;

    open(F, "< $conf" ) or die;
    $_ = <F>; chop;
    (/^#:GENSYS:$/) or die "Wrong configuration for model data file.";   
    while (<F>) {
       next if (/^\s+$/); chop;
       if (/^BEGIN\s*$/) {
          $setm = $setf = $sett = $seta = $setp = 0; next;
       } elsif (/^\s*model/) {
          (/^\s*model\s*=\s*\'([a-zA-Z]+)\'\s*$/) or die $_; $m = $1; $setm = 1;
       } elsif (/^\s*start/) {
          (/^\s*start\s*=\s*(\d+)\s*$/) or die $_; $f = $1; $setf = 1;
       } elsif (/^\s*stop/) {
          (/^\s*stop\s*=\s*(\d+)\s*$/) or die $_; $t = $1; $sett = 1; 
       } elsif (/^\s*annotations/) {
          (/^\s*annotations\s*=\s*\'([a-zA-Z|]+)\'\s*$/) or die $_;
          @a = split(/\|/, $1); $seta = 1;
       } elsif (/^\s*prefix/) {
          (/^\s*prefix\s*=\s*\"([\w\-\.]*)\"\s*$/) or die $_;
          $p = $1;  $setp = 1;
       } elsif (/^END\s*$/) {
          ($setm && $setf && $sett && $seta && $setp) or
                    die "Incomplete model specification.";
          if ($ref_model eq $m) {
              $start = $f; $stop = $t; @annotations = @a; $prefix = $p;
              close(F); return 1;
          } else {
              $setm = $setf = $sett = $setp = $seta = 0;
          }
       } else {
          die "Unrecognized field in model description. $_";
       }
    }
    # model organism not found 
    close(F); return 0;
}
   

if ($address=~/^\s*(\d+)\s*$/) {
    $address = $1;
} else {
    die "Address must be numeric.";
}

$from = $address -500; 
if ($from<$start) { $from = 1; } 
elsif ($from > $stop) { die "Address out of bounds."; }
$to = $address +500; 
if ($to > $stop) { $to = $stop; }

foreach $s (@annotations) { 
   if ($source =~ /^($s)$/) { $source =$1; $found = 1; last; } 
}
defined($found) or die "Invalid request for annotation data source.";

if ($format =~ /^(ps|pdf)$/) {
    $format = $1; 
} else {
    die "Invalid request for output format.";
}

if ($conservation =~ /^(agree|infocon|kkno|kunk)$/) { 
    $conservation = $1;
} else {
    die "Invalid request for conservation criterion.";
}

mkdir("$base",0775);
mkdir($tmp,0777) or die "mkdir $tmp: $!";
chdir($tmp) or die "chdir $tmp: $!";

sub END {
  plumber($tmp);
}

sub rm_all_files {
    opendir(DIR,".") or die "opendir: $!";
    @files = grep { -f $_ } readdir(DIR);
    @files = map {m!^([^/]+)$! ; $1} @files;  # untaint
    unlink @files or warn "unlink: $!";
    closedir DIR;
}

sub cleanup {
  my $s;

  if (defined($tmp)) {
    chdir($tmp) or die "chdir $tmp: $!";
    &rm_all_files;
    chdir($base) || die ;
    rmdir($tmp);
  }
}

!(system "mkalignment $from $to $prefix") or die "mkalignment $?";
!(system "mkconserved $from $conservation $prefix") or die "mkconsrved $?";
!(system "mklandmarks $from $to $source -r $prefix") or die "mklandmarks $?";
!(system "mkannotations $showgn $showpr $showpb $prefix") or die "mkannotations $?";

if ($format =~ /^ps$/) {
    print $q->header("application/postscript");
    open(F, "ps") or die;
    while (<F>) { print or die; }
    close(F) or die;
} else {
    print $q->header("application/pdf");
    !(system "ps2pdf ps") or die "ps2pdf failed $?";
    !(system "cat ps.pdf") or die "cat pdf file failed $?";
}
