#!/usr/local/bin/perl -w
use warnings;

(($#ARGV>=4) && ($#ARGV<=5)) || 
    die "Usage: $0 <ref_species> <xland_file> <gnlmark_file> <prlmark_file> <pblmark_file> <-r|-j> [<subtract>]\n";

my $species = shift;
my $xlandfile = shift;
my $gnlmarkfile = shift;
my $prlmarkfile = shift; 
my $pblmarkfile = shift;
my $format = shift;

my ($code, $from, $to, $name, $plus_one, $src, $link);
my ($color,$label,$subtract,$stop,$start);

($format=~/^-r$/) || ($format=~/^-j$/) || 
         die "Unrecognized format specification.";
if ($#ARGV==5) { $subtract = shift; } else { $subtract = 1; };
$subtract -= 1;
 

open(XLAND, "<$xlandfile\0") || 
    die "Could not open xlandmarks file $xlandfile.";

$_ = <XLAND>;
(/^\d+\s*\d+\s*$/) || die "Not an xland format file.";


open(GNLAND, ">$gnlmarkfile\0") or die "Could not open file $gnlmarkfile.";
open(PRLAND, ">$prlmarkfile\0") or die "Could not open file $prlmarkfile.";
open(PBLAND, ">$pblmarkfile\0") or die "Could not open file $pblmarkfile.";

print GNLAND "#:TUPLES:\n\n";
print PRLAND "#:TUPLES:\n\n";
print PBLAND "#:TUPLES:\n\n";

while (<XLAND>) {
   next if ((/^#/) || (/^\s*\n$/)); 
   if (($code)=/^\s*(PR)/) {
      (($from, $to, $name, undef, $plus_one, $src, $link) = 
       /^\s*PR\s+(\d+)\s+(\d+)\s+(\S+)\s+(\-?\d)\s+(\d+)\s+(\S+)\s+\(([^\)]*)\)\s*/)  ||
          die "Not an xland format file.$_";
   } elsif (($code)=/^\s*(GN)/) {
      (($from, $to, $name, undef, $src, $link) = 
       /^\s*GN\s+(\d+)\s+(\d+)\s+(\S+)\s+(\-?\d)\s+(\S+)\s+\(([^\)]*)\)\s*/) ||
          die "Not an xland format file. $_";
   } elsif (($code)=/^\s*(PB)/) {
      (($from, $to, $name, undef, $src, $link) = 
       /^\s*PB\s+(\d+)\s+(\d+)\s+(\S+)\s+(\-?\d)\s+(\S+)\s+\(([^\)]*)\)\s*/) ||
          die "Not an xland format file.$_";
   } else {
          die "Not an xland format file.$_";
   }
   
   $label = ($name eq "?") ? "PR":$name;
   if ($code =~ /GN/) {
     if ($format=~/^-r$/) {
       &print_landmark(GNLAND, $src, "${species}_orig", $from-$subtract, 
                               $to-$subtract,$label,$link);
     } else {
       $label .= " gene";
       &print_maj_landmark(GNLAND, $from, $to, $label, "LightChocolate End");
     }
   } elsif ($code =~ /PR/) { 
     if ($format =~ /^-r$/) {
       &print_landmark(PRLAND, $src, "${species}_orig", $from-$subtract,
                               $to-$subtract,$label,$link);
       &print_landmark(PRLAND, $src,"${species}_orig",$plus_one-$subtract,
                               $plus_one-$subtract,$label." +1",$link);
     } else {
       ($label = "predicted") if ($label =~ /PR/);
       &print_maj_landmark(PRLAND, $from, $to, $label." promoter", "Mint End");
       &print_maj_landmark(PRLAND, $plus_one, $plus_one, $label." promoter +1", "Mint End");
     }
   } else {
     if ($format =~ /^-r$/) { 
       &print_landmark(PBLAND, $src, "${species}_orig", $from-$subtract, 
                               $to-$subtract,$label,$link);  
     } else {
       &print_maj_landmark(PBLAND, $from, $to, $label." binding site", "Apple End");
     }
   }
}
close(PRLAND);
close(PBLAND);
close(GNLAND);
close(XLAND);

1;     # return value if successful


# ------  subroutines -------


sub print_landmark # file src orig start stop label link
{

   my ($file, $src,$orig,$start,$stop,$label) = @_;

   print $file "BEGIN landmark\n\tcontributor = '$src'\n\tBEGIN region\n";   
   print $file "\t\torigin = $orig\n\tstart = $start\n\tstop = $stop\n";
   print $file "\tEND region\n\tlabel = '\"$label\"'\n";
   ($link eq "") || print $file "\tnote = '$link'\n";
   print $file "END landmark\n\n"; 
}

sub print_maj_landmark # file start stop label color
{
   
   my ($file,$start,$stop,$label,$color) = @_;

   print $file "$start $stop ($label) $color\n";
}
